@@ -16,9 +16,10 @@ from pyocd.probe import aggregator
 from pyocd.probe.cmsis_dap_probe import CMSISDAPProbe
 
 # CMSIS DAP only due to https://github.com/mbedmicro/pyOCD/issues/877
-aggregator.PROBE_CLASSES = [
-    CMSISDAPProbe
-]
+if isinstance(aggregator.PROBE_CLASSES, list):
+    aggregator.PROBE_CLASSES = [
+        CMSISDAPProbe
+    ]
 
 from mcutk.debugger.base import DebuggerBase
 from mcutk.util import run_command
@@ -48,6 +49,9 @@ class PYOCD(DebuggerBase):
         return self.version not in (None, '')
 
     def set_board(self, board):
+        # workaround to clear the prefix of usbid
+        if board.usbid and ":" in board.usbid:
+            board.usbid = board.usbid.split(":")[-1]
         self._board = board
         self._check_overried_target(board.devicename)
 
@@ -81,7 +85,6 @@ class PYOCD(DebuggerBase):
         options = {}
         # pyocd 0.23: pre_reset to deal with low power mode
         if self.loose_version >= LooseVersion("0.23.0"):
-            options['connect_mode'] = 'pre-reset'
             if self._board and not self._board.name.startswith("evk"):
                 # workaround for none kinetis series
                 options['connect_mode'] = 'under-reset'
@@ -151,14 +154,14 @@ class PYOCD(DebuggerBase):
     def erase(self, **kwargs):
         """Mass erase flash."""
 
-        if self.loose_version >= LooseVersion("0.25.0"):
+        if self.loose_version >= LooseVersion("0.25.0") or 'dev' in self.version:
             command = "pyocd erase --mass -v --no-wait"
         else:
             command = "pyocd erase --mass-erase -v"
 
         command += " " + self._build_cmd_args(options={
             "resume_on_disconnect": False,
-            "connect_mode": 'pre-reset'
+
         })
         logging.info(command)
         return run_command(command, timeout=120)[0]
@@ -213,7 +216,7 @@ class PYOCD(DebuggerBase):
             port = board.gdbport
 
         if self.loose_version >= LooseVersion("0.13.0"):
-            command = "pyocd gdbserver --trust-crc"
+            command = "pyocd gdbserver --no-wait" # --trust-crc
             if port:
                 command += " --port %s" % port
         else:
@@ -222,7 +225,22 @@ class PYOCD(DebuggerBase):
             if port:
                 command += " -p %s" % port
 
-        command += " " + self._build_cmd_args(board)
+        if kwargs.get('erase'):
+            command += " --erase %s" % kwargs.get("erase")
+
+        if kwargs.get("script"):
+            command += " --script %s" % kwargs.get("script")
+
+        if kwargs.get("core") and self.loose_version >= LooseVersion("0.27.0"):
+            command += " --core %s" % kwargs.get("core")
+
+        options = self._get_default_session_options()
+        # set console to disable telent server
+        if self.loose_version >= LooseVersion("0.23.0"):
+            options['semihost_console_type'] = 'console'
+            options['keep_unwritten'] = 'false'
+
+        command += " " + self._build_cmd_args(board, options)
         return command
 
     def flash(self, filepath, **kwargs):
@@ -233,7 +251,7 @@ class PYOCD(DebuggerBase):
             addr: being the integer starting address for the bin file.
         """
         addr = kwargs.get('addr')
-        filepath = filepath.replace("\\", "/")
+        filepath = filepath.replace("/", "/")
         options = self._get_default_session_options()
         session = self.get_session(options=options)
         if session is None:
@@ -254,12 +272,30 @@ class PYOCD(DebuggerBase):
 
     def gdb_init_template(self):
         """Return default pypcd gdb commands."""
+        commands = [
+            "set remotetimeout 10",
+            "target remote localhost:{gdbport}",
+            # imxrt series, safe to reset and halt chip
+            # pyocd version < 0.28, the monitor reset will do reset_and_halt
+            # but when version >= 0.28, reset just reset core without halt
+            # to compatible with old version, we reset twice.
+            # It's safe to remove "monitor reset" in future if we deprecated
+            # pyocd < 0.28
+            "monitor reset --halt" if self.loose_version > LooseVersion("0.27.3") else "monitor reset",
+            "monitor halt",
+            "load",
+        ]
 
-        return ("set tcp connect-timeout 10\n"
-                "target remote localhost: {gdbport}\n"
-                "load\n"
-                "mon reset\n"
-                "q\n")
+        if self._board and self._board.sp:
+            commands.append("x {sp}")
+            commands.append("set $sp=$__")
+
+        if self._board and self._board.pc:
+            commands.append("x {pc}")
+            commands.append("set $pc=$__")
+
+        commands.append("q")
+        return "\n".join(commands)
 
     def __str__(self):
         return "pyocd-%s" % self.version
