@@ -16,18 +16,21 @@ except ImportError:
     import configparser
 
 from mcutk import util
-from mcutk.exceptions import CodeSyncError, BranchError, ProcessTimeout
+from mcutk.exceptions import ProcessTimeout
 
 
 bin_path = os.path.join(os.path.dirname(os.path.abspath(__file__)), 'bin')
 
+class GitCommandError(Exception):
+    pass
 
 def _get_repo_url(path):
-    ret, output = util.run_command('git config --get remote.origin.url', cwd=path, stdout=True, shell=True)
-    if ret != 0:
-        logging.warning('Not a valid git repository.')
-        return None
+    try:
+        subprocess.check_call('git rev-parse --is-inside-work-tree', cwd=path, shell=True)
+    except subprocess.CalledProcessError:
+        pass
 
+    ret, output = util.run_command('git config --get remote.origin.url', cwd=path, stdout=True, shell=True)
     return output.replace('\\n', '').strip()
 
 
@@ -56,9 +59,10 @@ class Repo(object):
     @classmethod
     def frompath(cls, path):
         """Return a repo object from specific path."""
-
+        if not os.path.exists(path):
+            return
         url = _get_repo_url(path)
-        if url:
+        if url is not None:
             return cls(url, path)
 
     @classmethod
@@ -86,7 +90,7 @@ class Repo(object):
     def __str__(self):
         return "Repo(name=%s, dir=%s)"%(self.repo_name, self.repo_dir)
 
-    def __exec(self, cmd, cwd=None, t=3600, retry=2):
+    def __exec(self, cmd, cwd=None, timeout=3600, retry=2, ignore_errors=False):
         if cwd is None:
             cwd = self.repo_dir
         error = None
@@ -94,7 +98,10 @@ class Repo(object):
             error = None
             logging.info(cmd)
             try:
-                noerror, output = util.run_command(cmd, cwd=cwd, stdout=True, timeout=t)
+                noerror, output = util.run_command(cmd, cwd=cwd, stdout=True, timeout=timeout)
+                if ignore_errors:
+                    return output
+
                 if noerror == 0:
                     break
                 # try to remove index.lock
@@ -102,11 +109,11 @@ class Repo(object):
                     self.remove_index_lock(self.repo_dir)
             except ProcessTimeout as e:
                 error = e
-            logging.info("retry again..")
+            logging.warning("retry again..")
         else:
-            raise CodeSyncError("Run command {0} failure".format(cmd))
+            raise GitCommandError(cmd)
 
-        if error:
+        if not ignore_errors and error:
             raise
         return output
 
@@ -116,7 +123,6 @@ class Repo(object):
             return False
 
         url_c = _get_repo_url(self.repo_dir)
-
         if not url_c:
             return False
 
@@ -138,10 +144,10 @@ class Repo(object):
                             url_c, self.repo_url)
             return False
 
-    def exec_cmd(self, cmd, cwd=None, timeout=3600, retry=2):
+    def exec_cmd(self, *args, **kwargs):
         """Execute a command."""
 
-        return self.__exec(cmd, cwd, timeout, retry)
+        return self.__exec(*args, **kwargs)
 
     def exec_cmds(self, cmds):
         """Execute a list of commands."""
@@ -157,7 +163,7 @@ class Repo(object):
         if not repo_dir:
             repo_dir = self.repo_dir
 
-        return self.__exec("git log -n 1", cwd=repo_dir, t=1)
+        return self.__exec("git log -n 1", cwd=repo_dir, timeout=1)
 
     def get_branch_name(self, repo_dir=None):
         """Return branch name.
@@ -166,7 +172,7 @@ class Repo(object):
         """
         if not repo_dir:
             repo_dir = self.repo_dir
-        return self.__exec("git rev-parse --abbrev-ref HEAD", cwd=repo_dir, t=1).replace("\n", "").strip()
+        return self.__exec("git rev-parse --abbrev-ref HEAD", cwd=repo_dir, timeout=2, ignore_errors=True).replace("\n", "").strip()
 
     def get_submodules(self, repo_dir=None):
         """Return a dictionary about all submodules.
@@ -217,15 +223,14 @@ class Repo(object):
 
         try:
             self.exec_cmd(cmd)
-        except CodeSyncError:
+        except GitCommandError:
             Repo.remove_index_lock(self.repo_dir)
             self.exec_cmd('git submodule deinit -f .')
             self.exec_cmd(cmd)
 
-
     def get_head_hash(self):
         """Return commit hash from head."""
-        return self.__exec("git rev-parse HEAD", t=1).replace("\n", "").strip()
+        return self.__exec("git rev-parse HEAD", timeout=1, ignore_errors=True).replace("\n", "").strip()
 
     def get_history_abbrev(self, since="1.weeks"):
         """Useful options for git log --pretty=format
@@ -276,7 +281,22 @@ class Repo(object):
 
         return branches
 
-    def clone(self, clone_path='.', branch_name=None, foldername=None):
+    @staticmethod
+    def quick_clone(url, clone_path, branch_name, foldername=None):
+        repo = Repo.fromurl(url, clone_path)
+        if foldername:
+            repo.repo_dir = os.path.join(clone_path, foldername)
+        else:
+            foldername = repo.repo_name
+
+        util.run_command('git init %s' % foldername, cwd=clone_path, need_raise=True)
+        repo.exec_cmds([
+            'git config remote.origin.url %s' % url,
+            'git config remote.origin.fetch "+refs/heads/*:refs/remotes/origin/*"'
+        ])
+        return repo
+
+    def clone(self, clone_path='.', branch_name=None, foldername=None, quick_clone=True):
         """Clone specific branch to specific directory.
 
         Arguments:
@@ -296,7 +316,6 @@ class Repo(object):
                 near the tips of allbranches. If you want to clone submodules shallowly also pass
                 --shallow-submodules.
 
-
             --branch <name>
             -b <name>
                 Instead of pointing the newly created HEAD to the branch pointed to by the cloned
@@ -306,6 +325,11 @@ class Repo(object):
         """
         logging.info("prepare to clone repo")
 
+        if quick_clone:
+            repo = Repo.quick_clone(self.repo_url, clone_path, branch_name, foldername)
+            self.repo_dir = repo.repo_dir
+            return self.repo_dir
+
         if self.repo_dir and clone_path == '.':
             clone_path = os.path.dirname(self.repo_dir)
 
@@ -327,7 +351,7 @@ class Repo(object):
 
         if ret != 0:
             logging.error("clone command return none-zero code!")
-            raise CodeSyncError("Failed to clone repo!")
+            raise GitCommandError("Failed to clone repo!")
 
         if self.repo_dir is None:
             self.repo_dir = os.path.join(clone_path, dir_name)
@@ -362,9 +386,7 @@ class Repo(object):
             shallow_fetch {bool} -- do shallow fetch (default: {False})
 
         Raises:
-            BranchError -- When branch is not exists
-            CodeSyncError -- When any git operation failed.
-
+            GitCommandError -- When any git operation failed.
 
         Returns:
             bool -- success or not
@@ -382,7 +404,7 @@ class Repo(object):
 
             self.__exec('git reset --hard', retry=1)
             self.__exec('git clean -fxd', retry=1)
-        except CodeSyncError:
+        except GitCommandError:
             if os.name == "nt":
                 util.change_folder_security_win(self.repo_dir)
             Repo.remove_index_lock(self.repo_dir)
@@ -392,8 +414,8 @@ class Repo(object):
 
         try:
             self.__exec(fetch_command, retry=4)
-        except CodeSyncError:
-            raise BranchError("Fetch Branch Error!")
+        except GitCommandError:
+            raise GitCommandError("Fetch Branch Error!")
 
         self.__exec('git checkout -f %s'%branch_name)
         self.__exec('git merge origin/%s'%branch_name)
